<?php

namespace App\Console\Commands;

use App\Models\Pub;
use App\Models\Sdoc;
use Illuminate\Console\Command;

use MicrosoftAzure\Storage\File\FileRestProxy;
use MicrosoftAzure\Storage\Common\ServiceException;
use MicrosoftAzure\Storage\File\FileSharedAccessSignatureHelper;
use MicrosoftAzure\Storage\Common\Internal\Resources;

class process_sdocs extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'process_sdocs';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Check for new sdoc documents';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle() {
        try {
            $endpointSuffix = env('AZURE_STORAGE_SUFFIX');
            $accountName = env('AZURE_STORAGE_ACCOUNT');
            $key = env('AZURE_STORAGE_KEY');
            $shareName = env('AZURE_STORAGE_SHARE');

            $connectionString = sprintf("DefaultEndpointsProtocol=https;AccountName=%s;AccountKey=%s;EndpointSuffix=%s", $accountName, $key, $endpointSuffix);
            $this->fileService = FileRestProxy::createFileService($connectionString);

            $files = $this->getFiles();
            // var_dump($files);
            // die();

            // $shares = $this->fileService->listShares();
            // var_dump($shares);
            // die();

            // $testFiles = array('FINSTATEMENT_HIL001_2001107701_0000000000_0000000006_0000000000_0000000000_0000000000_20200415_20191105051526.pdf', 'FINSTATEMENT_HIL001_2001107701_0000000000_0000000005_0000000000_0000000000_0000000000_20200311_20191105051526.pdf', 'FINSTATEMENT_HIL001_2001118402_0000000000_0000000004_0000000000_0000000000_0000000000_20200301_20191105051526.pdf', 'FINSTATEMENT_HIL001_2001118402_0000000000_0000000003_0000000000_0000000000_0000000000_20200131_20191105051526.pdf', 'FINSTATEMENT_HIL001_2001116400_0000000000_0000000002_0000000000_0000000000_0000000000_20200212_20191105051526.pdf', 'FINSTATEMENT_HIL001_2001116400_0000000000_0000000001_0000000000_0000000000_0000000000_20200121_20191105051526.pdf');

            // foreach ($testFiles as $test) {
            //     $fileClient->createFile($shareName, $test, 0);
            // }

            // $properties = $fileClient->getFileMetadata($shareName, $fileName);
            // var_dump($properties);

            // $helper = new FileSharedAccessSignatureHelper($accountName, $key);

            // $sasToken = $helper->generateFileServiceSharedAccessSignatureToken(Resources::RESOURCE_TYPE_FILE, "$shareName/$fileName", 'r', '2020-12-01T08:30:00Z');
            // $url = sprintf("https://%s.file.%s/%s/%s?%s", $accountName, $endpointSuffix, $shareName, $fileName, $sasToken);
            // echo $url;

            // $fileClient->setFileMetadata('sequeltest', 'test.pdf', array('processed' => true));           
        } catch (ServiceException $e) {
            $code = $e->getCode();
            $error_message = $e->getMessage();
            echo $code . ": " . $error_message . PHP_EOL;

            return;
        }


        foreach($files AS $file) {
            $filename = $file->getName();

            try {
                $details = $this->parseFilename($filename);

                // Check if it has been processed
                $processed = Sdoc::where('filename', $details->filename)->exists();
                if ($processed) {
                    throw new \Exception('Already processed:' . $details->filename);
                }

                // Lookup pub
                $pub = Pub::where('account_no', $details->account)->first();
                if (!$pub) {
                    throw new \Exception('Pub doesnt exist: ' . $details->account);
                }


                echo $pub->id;
                var_dump($details);
                echo PHP_EOL;

                $sdoc = new Sdoc;
                $sdoc->pub_id = $pub->id;
                $sdoc->invoice = $details->invoice;
                $sdoc->filename = $details->filename;
                $sdoc->date = date('Y-m-d', $details->date);
                $sdoc->save();

                $this->fileService->setFileMetadata($shareName, $details->filename, array('processed' => true));
            } catch (\Exception $e) {
                echo $e->getMessage() . PHP_EOL;
            }
        }
    }

    private function getFiles() {
        $shareName = env('AZURE_STORAGE_SHARE');
        $result = $this->fileService->listDirectoriesAndFiles($shareName);
        return $result->getFiles();
    }

    private function parseFilename($filename) {
        $basename = basename($filename, '.pdf');

        if ($basename === $filename) {
            throw new \Exception('Invalid file type: ' . $filename);
        }

        $parts = explode('_', $basename);

        if (count($parts) !== 10 && $parts[0] !== 'FINSTATEMENT') {
            throw new \Exception('Invalid file name: ' . $filename);
        }

        $details = new \stdClass();
        $details->filename = $filename;
        $details->account = $parts[2];
        $details->invoice = ltrim($parts[4], '0');
        $details->date = strtotime($parts[8]);

        return $details;
    }
}
