<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\StockController;
use App\Http\Controllers\Controller; 
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Maatwebsite\Excel\Facades\Excel;
use App\Mail\Order as OrderMail;
use App\Mail\Confirm as ConfirmMail;
use App\Mail\NonIntegratedOrder as NonIntegratedOrderMail;
use App\Exports\NonIntegratedOrder;
use App\Exports\HistoryExport;
use Illuminate\Http\Request;
use App\Models\Product;
use App\Models\Supplier;
use App\Models\Delivery;
use App\Models\Pub;
use App\Models\Stock;
use App\Models\Order;
use App\Models\Suborder;
use App\User;
use Session;
use Auth;
use PDF;
use Carbon;

class OrderController extends Controller
{   

   

    //add product to order
    public function add_product(Request $request){
           
           $pub_id = $request->pub_id;
           $user_id = Auth::user()->id;
           if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }
            if(!$this->check_permission($pub_id, $user_id, 'order')){
               return response(["message" => "Permission denied!"], 400);
              } 
           $order = $this->get_order($pub_id);
           $stock = $this->get_stock($pub_id);
           $data = json_decode($request->order);

           if (!$data || !is_iterable($data) || !count($data)) {
            return response(["message" => "Missing order details"], 400);
           }

           if($request->notes){
              $order->delivery_note = $request->notes;
              $order->save();
             }

           foreach($data as $product){
                   $product_found =  DB::table('order_product')
                              ->where('product_id', $product->product_id)
                              ->where('order_id', $order->id)
                              ->first();

                   $stock_product = DB::table('product_stock')
                                   ->where('stock_id', $stock->id)
                                   ->where('product_id', $product->product_id)
                                   ->first();
                   $stock_qty = 0;               
                   if($stock_product){
                      $stock_qty = DB::table('product_stock')
                                   ->where('stock_id', $stock->id)
                                   ->where('product_id', $product->product_id)
                                   ->first()->qty;
                   }                           
                    if($product_found){
                       DB::table('order_product')
                          ->where('product_id', $product->product_id)
                          ->where('order_id', $order->id)
                          ->update([
                                    'qty' => $product->quantity,
                                    'stock' => $stock_qty,
                                    'updated_at' => now(),
                                   ]);
                      }


                    if(!$product_found){
                        DB::table('order_product')->insert([
                            'product_id' => $product->product_id,
                            'order_id' => $order->id,
                            'qty' => $product->quantity,
                            'stock' => $stock_qty,
                            'supplier_id' => $product->supplier_id,
                            'created_at' => now(),
                            'updated_at' => now(),
                                 ]);
                      }


                 }//end of foreach
           
 
           $this->get_order_processed($order);
           $is_sent = 0;
           $confirmed_order = array();
           $confirmed_order['id'] = $order->id;
           $confirmed_order['delivery_date'] = $order->delivery_date;
           $confirmed_order['sub_orders'] = $this->get_all_sub_order_products($order, $is_sent); 
           
           return response(["data" => $confirmed_order]);

    }//end of confirm 


    

    public function sendToSupplier($orderID, $supplierID = null, Request $request) {
           $pub_id = $request->pub_id;
           $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }
            if(!$this->check_permission($pub_id, $user_id, 'order')){
               return response(["message" => "Permission denied!"], 400);
              }
      $resource = Suborder::where('order_id', $orderID);

      if ($supplierID) {
        $resource->where('supplier_id', $supplierID);
      }

      $suborders = $resource->get();

      foreach ($suborders AS $suborder) {
        if($suborder->supplier->integrated == 0) {
          return response(['Message' => 'Supplier is not integrated'], 400);
        }

        $sub_order = $this->get_sub_order_products($suborder);
        $pdf = PDF::loadView('mail.order_pdf', ['sub_order' => $sub_order]);
        $content = $pdf->download()->getOriginalContent();
        Storage::put('Order/'.$sub_order['ref'].'.pdf',$content);

        Mail::to($sub_order['supplier_email'])->send(new OrderMail($sub_order));

        $suborder->is_sent = 1;
        $suborder->save();
      }

      return response(['Message' => 'Email Sent']);
    }

    public function sendToUser($orderID, $supplierID, Request $request) {
      $pub_id = $request->pub_id;
      $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }
            if(!$this->check_permission($pub_id, $user_id, 'order')){
               return response(["message" => "Permission denied!"], 400);
              }
      $suborder = Suborder::where('order_id', $orderID)
                            ->where('supplier_id', $supplierID)
                            ->first();

      //create Excel file and store
      $sub_order = $this->get_sub_order_products($suborder);
      $excel = new NonIntegratedOrder($sub_order);
      Excel::store($excel, 'Order/'.$sub_order['ref'].'.xlsx');

      //send mail
      Mail::to(Auth::user()->email)->send(new NonIntegratedOrderMail($sub_order));

      if($suborder->supplier->integrated == 0) {
        $suborder->is_sent = 1;
        $suborder->save();
      }

      return response(['Message' => 'Email Sent']);
    }

    public function download($orderID, $supplierID, Request $request) {
      $pub_id = $request->pub_id;
      // $user_id = Auth::user()->id;
            // if(!$pub_id){
            //   return response(["message" => "Missing pub_id"], 400);
            // }
            // if(!$this->check_pub_user($pub_id, $user_id)){
            //   return response(["message" => "Access denied!"], 400);
            // }
            // if(!$this->check_permission($pub_id, $user_id, 'order')){
            //    return response(["message" => "Permission denied!"], 400);
            //   }
      $suborder = Suborder::where('order_id', $orderID)
                            ->where('supplier_id', $supplierID)
                            ->first();

      //create Excel file and store
      $sub_order = $this->get_sub_order_products($suborder);
      $excel = new NonIntegratedOrder($sub_order);
      return Excel::download($excel, $sub_order['ref'].'.xlsx');
    }


    private function get_all_sub_order_products($order, $is_sent){
           $sub_orders = array();
           $suborders = Suborder::where('order_id', $order->id)
                                 ->where('is_sent', $is_sent)
                                 ->where('is_cancelled', 0)
                                 ->get();

           $suborders->map(function($suborder)use(&$sub_orders){
              $sub_order_obj = array();
              $sub_order_product = array();
              $sub_order_obj['sub_order_id'] =  $suborder->id;
              $sub_order_obj['ref'] =  $suborder->ref;
              $sub_order_obj['supplier_id'] =  $suborder->supplier->id;
              $sub_order_obj['supplier'] = $suborder->supplier->name;
              $sub_order_obj['integrated'] = !!$suborder->supplier->integrated;
              $sub_order_obj['delivery_note'] = $this->get_delivery_note($suborder->order, $suborder->order->pub->id, $suborder->supplier->id);
              $suborder->products->map(function($product)use($suborder, &$sub_order_product){
                  $qty = DB::table('product_suborder')
                            ->where('suborder_id', $suborder->id)
                            ->where('product_id', $product->id)
                            ->first()->qty;
                  if($qty >= 1){
                      $product_obj = array();
                      $product_obj['product_id'] = $product->id;
                      $product_obj['qty'] = $qty;
                      $sub_order_product[] = $product_obj;
                    }//end of if qty
                });
              $sub_order_obj['products'] = $sub_order_product;
              $sub_orders[] = $sub_order_obj;
          });

           return $sub_orders;

    }

     private function get_sub_order_products($suborder){
              $sub_order = array();
              $sub_order_product = array();
              $sub_order['id'] =  $suborder->id;
              $sub_order['ref'] =  $suborder->ref;
              $sub_order['supplier'] = $suborder->supplier->name;
              $sub_order['supplier_code'] = $suborder->supplier->supplier_code;
              $sub_order['supplier_email'] = $suborder->supplier->email;
              $sub_order['pub_email'] = $suborder->order->pub->email;
              $sub_order['pub_contact_person'] = $suborder->order->pub->contact_person;
              $sub_order['pub_name'] = $suborder->order->pub->name;
              $sub_order['pub_address'] = $suborder->order->pub->address_line1." ".$suborder->order->pub->address_line2.", ".$suborder->order->pub->city.", ".$suborder->order->pub->postcode.", ".$suborder->order->pub->country;
              $sub_order['date'] =  $suborder->order->updated_at;
              $sub_order['delivery_note'] = $this->get_delivery_note($suborder->order, $suborder->order->pub->id, $suborder->supplier->id);
              if(!$sub_order['delivery_note']){ $sub_order['delivery_note'] = "Not given"; }
              $suborder->products->map(function($product)use($suborder, &$sub_order_product){
                  $qty = DB::table('product_suborder')
                            ->where('suborder_id', $suborder->id)
                            ->where('product_id', $product->id)
                            ->first()->qty;
                  if($qty >= 1){
                      $product_obj = array();
                      $product_obj['id'] = $product->id;
                      $product_obj['title'] = $product->title;
                      $product_obj['qty'] =$qty;
                      $sub_order_product[] = $product_obj;
                  }//end of if qty
                });
              $sub_order['products'] = $sub_order_product;
              
           return $sub_order;
    }


    private function get_order_processed($order){
      $order->is_confirmed = 1;
      $order->save();


      ///////////////--------------
      $products = DB::table('order_product')
                     ->where('order_id', $order->id)
                     ->get();
      $order_product = array();
      $sub_orders = array();
      //map through each supplier and match if product is found
      $suppliers = Supplier::where('is_active', 1)->get();
      $suppliers->map(function($supplier)use($products, &$sub_orders, $order){
                  $supplier_product_obj = array();
                  $supplier_obj = array();
                  $products->map(function($product)use($supplier, &$supplier_product_obj, $order){
                            $product_obj = array();
                            //for each product
                            ///////////----------------
                            if($supplier->id == $product->supplier_id){
                               $suborder = $this->get_suborder($order, $supplier);
                               $product_obj['id'] = $product->product_id;
                               $product_obj['title'] = Product::where('id', $product->product_id)->first()->title;
                               $product_obj['qty'] = $product->qty;
                               $supplier_product_obj[] = $product_obj;
                               //insert product_sub_order here
                               $duplicate = DB::table('product_suborder')
                                               ->where('product_id', $product->product_id)
                                               ->where('suborder_id', $suborder->id)
                                               ->first();
                               if(!$duplicate){ 
                                   DB::table('product_suborder')->insert([
                                      'suborder_id' => $suborder->id,
                                      'product_id' => $product->product_id,
                                      'qty' => $product->qty,
                                      'stock' => $product->stock,           
                                   ]);
                                  }   
                              }                              
                           });
                  
                  $supplier_obj['products'] = $supplier_product_obj;
                  $sub_orders[] = $supplier_obj;
                  //return $sub_orders;
                  //dd($supplier_product_obj);
              });//end of supplier map

    }//end of method

    private function get_suborder(Order $order, Supplier $supplier){
           $suborder = Suborder::where('order_id', $order->id)
                                ->where('supplier_id', $supplier->id)
                                ->first();
           if(!$suborder){
               $week = Carbon::now()->diffInWeeks($order->pub->created_at); 
               $now = Carbon::now();
               $date = $now->year.$now->month.$now->day; 
               $suborder = new Suborder;
               $suborder->order_id = $order->id;
               $suborder->supplier_id = $supplier->id;
               $suborder->ref = 'wk_'.$week.'_'.$date.'_'.$supplier->REF.'_'.$supplier->supplier_code;
               $suborder->save();
              }
            return $suborder;  
        
    }

     
     
     

     private function get_delivery_note($order, $pub_id, $supplier_id){
            $delivery_note = $order->delivery_note;
            if($delivery_note){
               return $delivery_note;
            }
            $delivery= Delivery::where('pub_id', $pub_id)->where('supplier_id', $supplier_id)->first();
            if(!$delivery){
              $delivery_note = "No delivery note was given.";
              }else{
                $delivery_note = $delivery->delivery_note;
              }
            return $delivery_note;  
       }

     
     public function history(Request $request){
            $pub_id = $request->pub_id;
            $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }
            if(!$this->check_permission($pub_id, $user_id, 'history')){
               return response(["message" => "Permission denied!"], 400);
              } 
            if($request->start || $request->end){
               $start = date($request->start);
               $end = date($request->end);

               $query = Order::where('pub_id', $pub_id)
                                ->where('is_confirmed', 1);

                if ($start) {
                  $query->where('updated_at', '>=', $start);
                }
                if ($end) {
                  $query->where('updated_at', '<=', $end);
                }
                                
                $orders = $query->orderBy('updated_at', 'desc')
                       ->get();

            }else{
                   $orders = Order::where('pub_id', $pub_id)
                                    ->where('is_confirmed', 1)
                                    ->orderBy('updated_at', 'desc')
                                    ->get();                

            }

            if ($orders->isEmpty()) {
              return response(["message" => "No orders found"], 400);
            }
            
           
            $history = array();
            
            $orders->map(function($order)use(&$history, $request){
              $history_obj = array();
              $history_obj['id'] = $order->id;
              $history_obj['order_date'] = $order->updated_at;
              if ($request->full && filter_var($request->full, FILTER_VALIDATE_BOOLEAN)) {
                $history_obj['order_details'] = $this->get_history_products($order);
              }
              $history[] = $history_obj;
            });
            
            return response($history);
     }


    public function single($order_id, Request $request){
        $pub_id = $request->pub_id;
        $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
        if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }
        if(!$this->check_permission($pub_id, $user_id, 'history')){
               return response(["message" => "Permission denied!"], 400);
              }         
    
        $order = Order::where('pub_id', $pub_id)
                        ->where('id', $order_id)
                        ->first();
                        
        $order_obj = array();
        $order_obj['id'] = $order->id;
        $order_obj['order_date'] = $order->updated_at;
        $order_obj['order_details'] = $this->get_history_products($order);

        return response($order_obj);
    }

    public function export(Request $request) {
        $pub_id = $request->pub_id;
        //$user_id = Auth::user()->id;

        if (!$pub_id) {
            return response(["message" => "Missing pub_id"], 400);
        }

        // if (!$this->check_pub_user($pub_id, $user_id)) {
        //     return response(["message" => "Access denied!"], 400);
        // }

        // if (!$this->check_permission($pub_id, $user_id, 'history')) {
        //     return response(["message" => "Permission denied!"], 400);
        // }

        $start = date($request->start);
        $end = date($request->end);

        $query = Order::where('pub_id', $pub_id);
        $query->where('is_confirmed', 1);

        if ($request->start) {
            $start = date('Y-m-d', strtotime($request->start));
            $query->where('updated_at', '>=', $start);
        }

        if ($request->end) {
            $end = date('Y-m-d', strtotime($request->end));
            $query->where('updated_at', '<=', $end);
        }

        $orders = $query->orderBy('updated_at', 'desc')->get();

        if ($orders->isEmpty()) {
            return response(["message" => "No orders found"], 400);
        }


        $timescale = "All time";
        if ($request->start) {
            $timescale = date('d/m/Y', strtotime($request->start)) . " - ";

            if ($request->end) {
                $timescale .= date('d/m/Y', strtotime($request->end));
            } else {
                $timescale .= date('d/m/Y');
            }
        } else {
            if ($request->end) {
                $timescale = 'Until ' . date('d/m/Y', strtotime($request->end));
            }
        }

        $stats = array(
            'orders' => 0,
            'suppliers' => array(),
            'products' => array(),
            'items' => 0
        );

        $orderObjs = [];
        foreach ($orders as &$order) {
            $order->lines = $this->get_history_products($order);

            // Does this order have products from out supplier
            if (isset($_GET['supplier'])) {
                $order->lines = array_filter($order->lines, function($line) {
                    return $line['supplier_id'] == $_GET['supplier'];
                });
                if (!count($order->lines)) {
                    unset($order);
                    continue;
                }
            }

            $stats['orders']++;
            foreach($order->lines AS $line) {
                array_push($stats['products'], $line['product_id']);
                array_push($stats['suppliers'], $line['supplier_id']);

                $stats['items'] += $line['quantity'];
            }

            $orderObjs[] = $this->get_export_order($order);
        }

        // Count number of unique products and suppliers
        $stats['suppliers'] = count(array_unique($stats['suppliers']));
        $stats['products'] = count(array_unique($stats['products']));

        $excel = new HistoryExport($stats, $timescale, $orderObjs);
        return Excel::download($excel, 'invoices.xlsx');
    }

    private function get_export_order($order) {
        $export = array();
        $export['id'] =  $order->id;
        $export['ref'] = $order->id;
        $export['pub_email'] = $order->pub->email;
        $export['pub_contact_person'] = $order->pub->contact_person;
        $export['pub_name'] = $order->pub->name;
        $export['pub_address'] = $order->pub->address_line1." ".$order->pub->address_line2.", ".$order->pub->city.", ".$order->pub->postcode.", ".$order->pub->country;
        $export['date'] =  $order->updated_at;
        $export['delivery_note'] =  $order->delivery_note ? $order->delivery_note :  "No delivery note was given.";
        $export['products'] = $order->lines;
        return $export;
    }




    private function get_history_products($order){
           $products = array();
           $suborders = Suborder::where('order_id', $order->id)
                                // ->where('is_sent', 1)
                                 ->where('is_cancelled', 0)
                                 ->get();

           $suborders->map(function($suborder) use (&$products){
              $suborder_products = $suborder->products->map(function($product) use ($suborder, &$products) {
                $qty = DB::table('product_suborder')
                            ->where('suborder_id', $suborder->id)
                            ->where('product_id', $product->id)
                            ->first()->qty;

                $stock = DB::table('product_suborder')
                             ->where('suborder_id', $suborder->id)
                             ->where('product_id', $product->id)
                             ->first()->stock; 

                $product_obj = array();
                $product_obj['product_id'] = $product->id;
                $product_obj['title'] = $product->title;
                $product_obj['supplier_id'] = $suborder->supplier->id;
                $product_obj['sent'] =  $suborder->is_sent;
                $product_obj['quantity'] =  $qty;
                $product_obj['qty'] =  $qty;
                $product_obj['stock'] =  $stock;

                $products[] = $product_obj;
              });
          });

          return $products;
    }



     private function get_order($pub_id){
        $order = Order::where('pub_id', $pub_id)
                       ->where('is_current', 1)
                       ->where('is_confirmed', 0)
                       ->first();
        
        if(!$order){
            $order = new Order;
            $order->user_id = Auth::user()->id;
            $order->pub_id = $pub_id;
            $order->save();
          }
           
        return $order;
     }//end of get_order

     private function get_stock($pub_id){
        $stock = Stock::where('pub_id', $pub_id)->where('is_active', 1)->first();
        if(!$stock){
           $stock = new Stock;
           $stock->pub_id = $pub_id;
           $stock->save();
          } 
        return $stock;
     }

     
     private function check_permission($pub_id, $user_id, $permission){
      //check if the user role is PubAdmin
      if(User::where('id', $user_id)->first()->role->title == 'PubAdmin'){
         return true;
      }
      //check if the user has the permission
      
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $allowed = false;
      $granted_permissions->map(function($granted)use(&$allowed, $permission){
         if($granted->permission == $permission){
          $allowed = true;
         } 
      }); 
        

      return $allowed;


   }//end of method    


   //this method will check if given user works for a given pub
    private function check_pub_user($pub_id, $user_id){
        $user_found = DB::table('pub_user')
                         ->where('pub_id', $pub_id)
                         ->where('user_id', $user_id )
                         ->where('is_active', 1)
                         ->first();
           
          if ($user_found) {
            return true;
          }else{
            return false;
          }
       
     }//end of method



    

}






