<?php

namespace App\Http\Controllers\Api;

use App\Http\Resources\UserCollection;
use App\Http\Resources\User as UserResource;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Models\Permission;
use App\User;
use Auth;
use Hash;

class PermissionController extends Controller
{

    
  public function get_users(Request $request){
     $pub_id = $request->pub_id;
     if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
      if(!$this->check_permission($pub_id, Auth::user()->id, 'admin')){
               return response(["message" => "Permission denied!"], 400);
            }


     // get all the users for given pub from pub_users
     
      $users = DB::table('pub_user')
                  ->where('pub_id', $pub_id)
                  ->get();
     

     //map through each user and create  
     $user_list = array();
     $users->map(function($pub_user)use(&$user_list, $pub_id){
      
     $user = User::where('id', $pub_user->user_id)->first();

     $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user->id)
                                ->get();
      
      $permission_list = array();
      $granted_permissions->map(function($permission)use(&$permission_list){
         $permission_list[] = $permission->permission; 
      }); 
    
      $user_obj = array();
      $user_obj['id'] = $user->id;
      $user_obj['name'] = $user->name;
      $user_obj['email'] = $user->email;
      $user_obj['role'] = $user->role->title;
      $user_obj['is_active'] = $pub_user->is_active;
      $user_obj['permissions'] = $permission_list;
      $user_list[] = $user_obj;


     });
       
      

      
      return response($user_list);  
    }//end of method






    public function create_user(Request $request){
     $user_id = Auth::user()->id;
     $pub_id = $request->pub_id;
     if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
     if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied! The user does not work for the requested pub"], 400);
            }
      if(!$this->check_permission($pub_id, $user_id, 'admin')){
               return response(["message" => "Permission denied!"], 400);
            }

       
      $data = json_decode($request->permissions);
      if (!$data || !is_iterable($data) || !count($data)) {
            return response(["message" => "Missing permission details"], 400);
           }    
      
      $user = User::where('email', $request->email)->first();

      if(!$user){
          $user = new User;
          $user->name = $request->name;
          $user->role_id = 4;
          $user->email = $request->email;
          $user->password = Hash::make('tempPass'.$pub_id);
          $user->save();
         }
      

      //insert pub_user
      //if the user is a active worker for the pub -> do nothing
      //if not active worker, has ever worked, if yes->update, if no->create   

      $active_worker = DB::table('pub_user')
                            ->where('pub_id', $pub_id)
                            ->where('user_id', $user->id)
                            ->where('is_active', 1)
                            ->first();
      
      if(!$active_worker){
         $ever_worked = DB::table('pub_user')
                            ->where('pub_id', $pub_id)
                            ->where('user_id', $user->id)
                            ->where('is_active', 0)
                            ->first();
         if($ever_worked){
            DB::table('pub_user')
                ->where('pub_id', $pub_id)
                ->where('user_id', $user->id)
                ->update([
                  'is_active' => 1
                ]);
         }else{
                DB::table('pub_user')->insert([
                  'pub_id' => $pub_id,
                  'user_id' => $user->id
                ]);
            }//else ever worked
      } //not active worker           

      //insert permission
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user->id)
                                ->get();
      
      $permissions = Permission::get();
      
      $permissions->map(function($permission)use($granted_permissions, $user, $pub_id, $data){
        //is_granted
        //is_input
        //is_granted BUT not is_input ---> delete from permission_user table
        //is_input BUT not is_granted ---> insert into permission_user table
        //is_input but AND is_granted ---> do nothing
        $is_granted = false;
        $is_input = false;
        foreach($granted_permissions as $granted){
            if($granted->permission_id == $permission->id){
                $is_granted = true;
            }
        }
        if(in_array($permission->title, $data)){
            $is_input = true;
        }
        if($is_granted && !$is_input){
           DB::table('permission_user')
              ->where('pub_id', $pub_id)
              ->where('user_id', $user->id)
              ->where('permission_id', $permission->id)
              ->delete(); 
        }
        if(!$is_granted && $is_input){
           DB::table('permission_user')->insert(
            [
              'pub_id' => $pub_id,
              'user_id' => $user->id,
              'permission_id' => $permission->id,
              'permission' => $permission->title,
              'created_at' => now(),
              'updated_at' =>now()
            ]); 
        }
      });
      

      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user->id)
                                ->get();
      
      $permission_list = array();
      $granted_permissions->map(function($permission)use(&$permission_list){
         $permission_list[] = $permission->permission; 
      }); 


     
      $user_obj = array();
      $user_obj['id'] = $user->id;
      $user_obj['name'] = $user->name;
      $user_obj['email'] = $user->email;
      $user_obj['role'] = $user->role->title;
      $user_obj['permissions'] = $permission_list;
      $user_obj['password'] = 'tempPass'.$pub_id;

      
      return response($user_obj);  
    }//end of method


    public function edit_permission(Request $request){
     $is_active = 1;
     $pub_id = $request->pub_id;
     $user_id = $request->user_id;
     $is_active = $request->active;
     if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
     if(!$user_id){
              return response(["message" => "Missing user_id"], 400);
            }
      
      if(!$this->check_pub_user($pub_id, Auth::user()->id)){
            return response(["message" => "Access denied! The user does not work for the requested pub"], 400);
            }
      if(!$this->check_permission($pub_id, Auth::user()->id, 'admin')){
               return response(["message" => "Permission denied!"], 400);
            }      
     $data = json_decode($request->permissions);
     if (!$data || !is_iterable($data) || !count($data)) {
            return response(["message" => "Missing permission details"], 400);
           }
      

      $active_worker = DB::table('pub_user')
                            ->where('pub_id', $pub_id)
                            ->where('user_id', $user_id)
                            ->update([
                              'is_active' => $is_active
                            ]);
      
      


      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $permissions = Permission::get();
      
      $permissions->map(function($permission)use($granted_permissions, $user_id, $pub_id, $data){
        //is_granted
        //is_input
        //is_granted BUT not is_input ---> delete from permission_user table
        //is_input BUT not is_granted ---> insert into permission_user table
        //is_input but AND is_granted ---> do nothing
        $is_granted = false;
        $is_input = false;
        foreach($granted_permissions as $granted){
            if($granted->permission_id == $permission->id){
                $is_granted = true;
            }
        }
        if(in_array($permission->title, $data)){
            $is_input = true;
        }
        if($is_granted && !$is_input){
           DB::table('permission_user')
              ->where('pub_id', $pub_id)
              ->where('user_id', $user_id)
              ->where('permission_id', $permission->id)
              ->delete(); 
        }
        if(!$is_granted && $is_input){
           DB::table('permission_user')->insert(
            [
              'pub_id' => $pub_id,
              'user_id' => $user_id,
              'permission_id' => $permission->id,
              'permission' => $permission->title,
              'created_at' => now(),
              'updated_at' =>now()
            ]); 
        }
      });
      

      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $permission_list = array();
      $granted_permissions->map(function($permission)use(&$permission_list){
         $permission_list[] = $permission->permission; 
      }); 
      
      $user = User::where('id', $user_id)->first();
      $user_obj = array();
      $user_obj['id'] = $user->id;
      $user_obj['name'] = $user->name;
      $user_obj['email'] = $user->email;
      $user_obj['role'] = $user->role->title;
      $user_obj['permissions'] = $permission_list;
      $user_obj['active'] = DB::table('pub_user')
                            ->where('pub_id', $pub_id)
                            ->where('user_id', $user_id)
                            ->first()->is_active;
      

      return response(["user" => $user_obj]);

    }//end of method


    
  
  


    private function check_permission($pub_id, $user_id, $permission){
      //check if the user role is PubAdmin
      if(User::where('id', $user_id)->first()->role->title == 'PubAdmin'){
         return true;
      }
      //check if the user has the permission
      
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $allowed = false;
      $granted_permissions->map(function($granted)use(&$allowed, $permission){
         if($granted->permission == $permission){
          $allowed = true;
         } 
      }); 
        

      return $allowed;


   }//end of method

    //this method will check if given user works for a given pub
    private function check_pub_user($pub_id, $user_id){
        $user_found = DB::table('pub_user')
                         ->where('pub_id', $pub_id)
                         ->where('user_id', $user_id )
                         ->where('is_active', 1)
                         ->first();
           
          if ($user_found) {
            return true;
          }else{
            return false;
          }
       
     }//end of method

}
