<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use GuzzleHttp\Client;
use App\Http\Resources\ProductCollection;
use App\Http\Resources\Product as ProductResource;
use App\Http\Resources\CategoryCollection;
use App\Http\Resources\Category as CategoryResource;
use App\Http\Resources\SubCategoryCollection;
use App\Http\Resources\SubCategory as SubCategoryResource;
use App\Http\Resources\FavouriteProduct;
use App\Models\Product;
use App\Models\Category;
use App\Models\SubCategory;
use App\Models\Pub;
use App\Models\Stock;
use App\User;
use Auth;

class ProductController extends Controller
{


    public function index(){
	    $products = Product::where('is_active', 1)->get();
	    return new ProductCollection($products);

    }


    public function categories(){
        $categories = Category::where('is_active', 1)->get();
        $all_categories = array();
        $categories->map(function($category)use(&$all_categories){
          $sub_categories = SubCategory::where('category_id', $category->id)->get();  
          $category_obj = array();
          $category_obj['id'] = $category->id;
          $category_obj['name'] = $category->title;
          $category_obj['placeholder_image'] = $category->image;
          $category_obj['children'] = new SubCategoryCollection($sub_categories);
          $all_categories[] = $category_obj; 
        });

        return response(["data" => $all_categories]);

    }


      //favourites product
      public function favourites(Request $request){
        $pub_id = $request->pub_id;
        $user_id = Auth::user()->id;
        if(!$pub_id){
            return response(["message" => "Missing pub_id"], 400);
            }
        if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
            }
        
        $products = DB::table('product_pub')->where('pub_id', $pub_id)->get();
        $favourites = array();
        
        foreach($products as $product){
          $product_obj = array();
          $product_obj['product_id'] = $product->product_id;
          $product_obj['par'] = $product->target_qty;
          $product_obj['item_order'] = $product->item_order;
          $product_obj['supplier_id'] = $product->supplier_id;
          $favourites[] = $product_obj;
          
        }
        return response(['data' => $favourites]);
    }



    //update target 
    public function add_favourites(Request $request){
           $pub_id = $request->pub_id;
           $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
               return response(["message" => "Access denied!"], 400);
              }
            if(!$this->check_permission($pub_id, $user_id, 'pub_edit')){
               return response(["message" => "Permission denied!"], 400);
              }  
           $data = json_decode($request->data);
           //return 'got it';

           if (!$data || !is_iterable($data) || !count($data)) {
            return response(["message" => "Missing favourites"], 400);
           }

           foreach($data as $product){
                   $check =  DB::table('product_pub')
                                ->where('product_id', $product->product_id)
                                ->where('pub_id', $pub_id)->first();              
                   if($check){
                      if($product->par == 0){
                            DB::table('product_pub')
                              ->where('product_id', $product->product_id)
                              ->where('pub_id', $pub_id)
                              ->delete();
                        }
                      if($product->par != 0){
                        
                          $updates = [
                                      'target_qty' => $product->par,
                                      'updated_at' => now(),
                                     ];

                          if (property_exists($product, 'item_order')) {
                            $updates['item_order'] = $product->item_order;
                          }

                          if (property_exists($product, 'supplier_id')) {
                            $updates['supplier_id'] = $product->supplier_id;
                          }

                          DB::table('product_pub')
                            ->where('product_id', $product->product_id)
                            ->where('pub_id', $pub_id)
                            ->update($updates);
                         }//if($product->par != 0)                 
                     }//if($check)
                   if(!$check){
                      DB::table('product_pub')->insert([
                          'product_id' => $product->product_id,
                          'pub_id' => $pub_id,
                          'target_qty' => $product->par,
                          'item_order' => property_exists($product, 'item_order') ? $product->item_order : 999,
                          'supplier_id' => property_exists($product, 'supplier_id') ? $product->supplier_id : null,
                          'created_at' => now(),
                          'updated_at' => now(),
                               ]);
                    }//end of else
                }//end for each 

                $products = DB::table('product_pub')->where('pub_id', $pub_id)->get();
                $favourites = array();
                
                foreach($products as $product){
                  $product_obj = array();
                  $product_obj['product_id'] = $product->product_id;
                  $product_obj['par'] = $product->target_qty;
                  $product_obj['item_order'] = $product->item_order;
                  $product_obj['supplier_id'] = $product->supplier_id;
                  $favourites[] = $product_obj;
                  
                }
                return response(['data' => $favourites]);   
    }//end of method 



   private function check_permission($pub_id, $user_id, $permission){
      //check if the user role is PubAdmin
      if(User::where('id', $user_id)->first()->role->title == 'PubAdmin'){
         return true;
      }
      //check if the user has the permission
      
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $allowed = false;
      $granted_permissions->map(function($granted)use(&$allowed, $permission){
         if($granted->permission == $permission){
          $allowed = true;
         } 
      }); 
        

      return $allowed;


   }//end of method    


   //this method will check if given user works for a given pub
    private function check_pub_user($pub_id, $user_id){
        $user_found = DB::table('pub_user')
                         ->where('pub_id', $pub_id)
                         ->where('user_id', $user_id )
                         ->where('is_active', 1)
                         ->first();
           
          if ($user_found) {
            return true;
          }else{
            return false;
          }
       
     }//end of method


    

}
