<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\DB;
use App\Http\Resources\PubCollection;
use App\Http\Resources\Pub as PubResource;
use App\Http\Resources\Feedback as FeedbackResource;
use App\Http\Resources\SupplierCollection;
use App\Http\Resources\Supplier as SupplierResource;
use App\Models\Pub;
use App\Models\Feedback;
use App\Models\Supplier;
use App\User;
use Hash;
use Auth;

class PubController extends Controller
{


    public function single(Request $request){
        $pub_id = $request->pub_id;
        $user_id = Auth::user()->id;
        if(!$pub_id){
          return response(["message" => "Missing pub_id"], 400);
        }
        if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
        }
		    $pub = Pub::where('id', $pub_id)->first();
		    return new PubResource($pub);

    }

    public function edit(Request $request){
            $pub_id = $request->pub_id;
            $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
            }
            if(!$this->check_permission($pub_id, $user_id, 'pub_edit')){
               return response(["message" => "Permission denied!"], 400);
            }

            $pub = Pub::where('id', $pub_id)->first();    
            if($request->eircode){
                $pub->update(['eircode' => $request->eircode]); 
            }
            if($request->account_no){
               $pub->update(['account_no' => $request->account_no]);  
            }
            if($request->HIL_payer_no){
               $pub->update(['HIL_payer_no' => $request->HIL_payer_no]);  
            }
            if($request->CBL_acc_no){
               $pub->update(['CBL_acc_no' => $request->CBL_acc_no]);  
            }
            return new PubResource($pub);
    }


    public function feedback(Request $request){

            if (!$request->comment || !$request->rating) {
                return response(["message" => "Invalid feedback"], 400);
            }

            $request->rating = intval($request->rating);
            if ($request->rating < 1 || $request->rating > 5) {
                return response(["message" => "Invalid rating"], 400);
            }

    
		    $feedback = new Feedback;
		    $feedback->user_id = Auth::user()->id;
		    $feedback->comment = $request->comment;
		    $feedback->rating = $request->rating;
		    $feedback->save();
		    
            return new FeedbackResource($feedback);

    }

    public function suppliers(){
        $suppliers = Supplier::where('is_active', 1)->get();
        return new SupplierCollection($suppliers);
    }

    public function pub_suppliers(Request $request){
        $pub_id = $request->pub_id;
        $user_id = Auth::user()->id;
        if(!$pub_id){
          return response(["message" => "Missing pub_id"], 400);
        }
        if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
            }

        $pub = Pub::where('id', $pub_id)->first();
        $suppliers = $pub->suppliers;

        return new SupplierCollection($suppliers);
    }

    public function add_suppliers(Request $request){
           $pub_id = $request->pub_id;
           $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
            }
           $supplier_id = $request->supplier_id;
           if(!$supplier_id){
              return response(["message" => "No supplier ID was given"], 400);
           }

           $check = DB::table('pub_supplier')
                        ->where('pub_id', $pub_id)
                        ->where('supplier_id', $supplier_id)
                        ->first();

            if(!$check){
               DB::table('pub_supplier')->insert([
                  'pub_id' => $pub_id,
                  'supplier_id' => $supplier_id
               ]); 
            }

            $pub = Pub::where('id', $pub_id)->first();
            $suppliers = $pub->suppliers;

            return new SupplierCollection($suppliers);            
    }


    public function delete_suppliers(Request $request){
           $pub_id = $request->pub_id;
           $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
            }
           $supplier_id = $request->supplier_id;
           if(!$supplier_id){
              return response(["message" => "No supplier ID was given"], 400);
           }

           DB::table('pub_supplier')
                        ->where('pub_id', $pub_id)
                        ->where('supplier_id', $supplier_id)
                        ->delete();

            $pub = Pub::where('id', $pub_id)->first();
            $suppliers = $pub->suppliers;

            return new SupplierCollection($suppliers);            
    }


   private function check_permission($pub_id, $user_id, $permission){
      //check if the user role is PubAdmin
      if(User::where('id', $user_id)->first()->role->title == 'PubAdmin'){
         return true;
      }
      //check if the user has the permission
      
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $allowed = false;
      $granted_permissions->map(function($granted)use(&$allowed, $permission){
         if($granted->permission == $permission){
          $allowed = true;
         } 
      }); 
        

      return $allowed;


   }//end of method    


    //this method will check if given user works for a given pub
    private function check_pub_user($pub_id, $user_id){
        $user_found = DB::table('pub_user')
                         ->where('pub_id', $pub_id)
                         ->where('user_id', $user_id )
                         ->where('is_active', 1)
                         ->first();
           
          if ($user_found) {
            return true;
          }else{
            return false;
          }
       
     }//end of method


    
    

}
