<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller; 
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Models\Product;
use App\Models\Pub;
use App\Models\Stock;
use App\Models\Order;
use App\Models\Suborder;
use App\User;
use Session;
use Auth;

class StockController extends Controller
{
    


    //current stock
      public function current(Request $request){
        $pub_id = $request->pub_id;
        $user_id = Auth::user()->id;
        if(!$pub_id){
          return response(["message" => "Missing pub_id"], 400);
        }
        if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }  
        if(!$this->check_permission($pub_id, $user_id, 'stock')){
               return response(["message" => "Permission denied!"], 400);
              } 
        $stock = $this->get_stock($pub_id);
        $products = $stock->products;
        $stok_product = array();
        
        $products->map(function($product)use(&$stock_product, $stock){
            $product_obj = array();
            $product_obj['product_id'] = $product->id;
            $product_obj['stock'] = DB::table('product_stock')
                                     ->where('stock_id', $stock->id)
                                     ->where('product_id', $product->id)
                                     ->first()->qty;
            $stock_product[] = $product_obj;
        });

       return response(['data' => $stock_product]);
    }  
    




    //update stock
    public function update_stock(Request $request){
           $pub_id = $request->pub_id;
           $user_id = Auth::user()->id;
            if(!$pub_id){
              return response(["message" => "Missing pub_id"], 400);
            }
            if(!$this->check_pub_user($pub_id, $user_id)){
              return response(["message" => "Access denied!"], 400);
            }
            if(!$this->check_permission($pub_id, $user_id, 'stock')){
               return response(["message" => "Permission denied!"], 400);
              } 

           $stock = $this->get_stock($pub_id);
           $data = json_decode($request->stock);

           if (!$data || !is_iterable($data) || !count($data)) {
            return response(["message" => "Missing stock details"], 400);
           }
           
           foreach($data as $product){
           $check =  DB::table('product_stock')
                        ->where('product_id', $product->product_id)
                        ->where('stock_id', $stock->id)
                        ->first(); 
           if($check){
               DB::table('product_stock')
                  ->where('product_id', $product->product_id)
                  ->where('stock_id', $stock->id)
                  ->update([
                            'qty' => $product->quantity,
                            'updated_at' => now(),
                           ]);
           }else{
                  DB::table('product_stock')->insert([
                      'product_id' => $product->product_id,
                      'stock_id' => $stock->id,
                      'qty' => $product->quantity,
                      'created_at' => now(),
                      'updated_at' => now(),
                           ]);
                }//end of else
           }//end of foreach
            

            $products = $stock->products;
            $stock_product = array();
              
            $products->map(function($product)use(&$stock_product, $stock){
                  $product_obj = array();
                  $product_obj['product_id'] = $product->id;
                  $product_obj['stock'] = DB::table('product_stock')
                                           ->where('stock_id', $stock->id)
                                           ->where('product_id', $product->id)
                                           ->first()->qty;                         
                  $stock_product[] = $product_obj;
              });

             return response([ "data" => $stock_product]);     
    }//end of method 



    //stock history
     public function history(Request $request){
        $pub_id = $request->pub_id;
        $user_id = Auth::user()->id;
        if(!$pub_id){
          return response(["message" => "Missing pub_id"], 400);
        }
        if(!$this->check_pub_user($pub_id, $user_id)){
            return response(["message" => "Access denied!"], 400);
        }
        if(!$this->check_permission($pub_id, $user_id, 'history')){
               return response(["message" => "Permission denied!"], 400);
              } 
        $stocks = Stock::where('pub_id', $pub_id)
                        ->where('is_active', 0)
                        ->orderBy('created_at', 'desc')
                        ->get();
        
        return response($stocks); 
    }


    private function get_stock($pub_id){
        $stock = Stock::where('pub_id', $pub_id)
                       ->where('is_active', 1)
                       ->first();
        if(!$stock){
           $stock = new Stock;
           $stock->pub_id = $pub_id;
           $stock->save();
          }
        return $stock;
     }



    private function check_permission($pub_id, $user_id, $permission){
      //check if the user role is PubAdmin
      if(User::where('id', $user_id)->first()->role->title == 'PubAdmin'){
         return true;
      }
      //check if the user has the permission
      
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $allowed = false;
      $granted_permissions->map(function($granted)use(&$allowed, $permission){
         if($granted->permission == $permission){
          $allowed = true;
         } 
      }); 
        

      return $allowed;


   }//end of method    


    //this method will check if given user works for a given pub
    private function check_pub_user($pub_id, $user_id){
        $user_found = DB::table('pub_user')
                         ->where('pub_id', $pub_id)
                         ->where('user_id', $user_id )
                         ->where('is_active', 1)
                         ->first();
           
          if ($user_found) {
            return true;
          }else{
            return false;
          }
       
     }//end of method





    
}

