<?php

namespace App\Http\Controllers;

use  Intervention\Image\Facades\Image; 
use App\Models\Category;
use Illuminate\Http\Request;
use Auth;

class CategoryController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
         return view('category.index', [
            'categories' => Category::orderBy('created_at', 'desc')->paginate(15)
        ]);
    }
    
    public function new(Request $request){
        if($request->isMethod('POST')){  
           
           $validatedData = $request->validate([
           'title' => 'required|unique:categories|max:255',
           'image' => 'sometimes|file|image|max:5000',
           ]);
           
           $category = new Category;
           $category->modified_by = Auth::user()->id;
           $category->title = $request->input('title');
           $category->save();  


           if($request->has('image')){
              $category->image = 'images/categories/'.$category->id."_".time().'.jpg';
              $category->save();
              $request->image->move('images/categories/', $category->id."_".time().'.jpg');
              $img = Image::make(public_path('images/categories/'.$category->id."_".time().'.jpg'))->crop(700, 700);
              $img->save();
           }


            return redirect()->route('category.single', [
                'category' => $category->id
            ]);
        }
        return view('category.new', [
            'category' => new Category,
        ]);
    }
    public function single(Category $category, Request $request){
        return view('category.single', [
            'category' => $category
        ]);
    }
    public function edit(Category $category, Request $request){
        if($request->isMethod('POST')){
           
           $validatedData = $request->validate([
           'title' => 'required|max:255',
           'image' => 'sometimes|file|image|max:5000',
           ]);
           
           $category->modified_by = Auth::user()->id;
           $category->title = $request->input('title');
           $category->save(); 


           if($request->has('image')){
              $category->image = 'images/categories/'.$category->id."_".time().'.jpg';
              $category->save();
              $request->image->move('images/categories/', $category->id."_".time().'.jpg');
              $img = Image::make(public_path('images/categories/'.$category->id."_".time().'.jpg'))->crop(700, 700);
              $img->save();
           }

      
            return redirect()->route('category.single', [
                'category' => $category->id
            ]);
        }
        return view('category.edit', [
            'category' => $category,
        ]);
    }
    public function delete(Category $category, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            $category->remove();
            return redirect()->route('category.index');
        }
        return view('category.delete', [
            'category' => $category
        ]);
    }   
}
