<?php

namespace App\Http\Controllers\Product;

use App\Http\Controllers\Controller;
use  Intervention\Image\Facades\Image; 
use App\Models\SubCategory;
use App\Models\Category;
use App\Models\Product;
use App\Models\Supplier;
use Illuminate\Http\Request;
use Auth;

class ProductController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
         return view('product.index', [
            'products' => Product::where('is_active', 1)->orderBy('title', 'asc')->paginate(15)
        ]);
    }

    
    public function index_desc(Request $request){
         return view('product.index_desc', [
            'products' => Product::where('is_active', 1)->orderBy('title', 'desc')->paginate(15),
            'all_products' => Product::where('is_active', 1)->get(),
        ]);
    }


    public function sku(Request $request){
         return view('product.index_desc', [
            'products' => Product::where('is_active', 1)->orderBy('SKU', 'desc')->paginate(15),
            'all_products' => Product::where('is_active', 1)->get(),
        ]);
    }


    public function brand_owner(Request $request){
         return view('product.index_desc', [
            'products' => Product::where('is_active', 1)->orderBy('brand_owner', 'desc')->paginate(15),
            'all_products' => Product::where('is_active', 1)->get(),
        ]);
    }


    
    public function new(Request $request){

           $validatedData = $request->validate([
           'sub_category_select' => 'required',
           'title' => 'required|max:255',
           'SKU' => 'required',
           'brand_owner' => 'required',
           'image' => 'sometimes|file|image|max:5000',
           ]);
           
           $product = new Product;
           $product->modified_by = Auth::user()->id;
           $product->sub_category_id = $request->input('sub_category_select');
           $product->supplier_id = $request->input('supplier_select');
           $product->title = $request->input('title');
           $product->SKU = $request->input('SKU');
           $product->brand_owner = $request->input('brand_owner');
           $product->description = $request->input('description');
           $product->purchase_UOM = $request->input('purchase_UOM');
           $product->save();  

           if($request->has('image')){
              $product->image = 'images/products/'.$product->id."_".time().'.jpg';
              $product->save();
              $request->image->move('images/products/', $product->id."_".time().'.jpg');
              $img = Image::make(public_path('images/products/'.$product->id."_".time().'.jpg'))->crop(700, 700);
              $img->save();
           } 



            return redirect()->route('product.single', [
                'product' => $product->id
            ]);
      
    }


    public function category_list(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'category_select' => 'required|max:255',
           ]);

          return view('product.new', [
            'product' => new Product,
            'sub_categories' => SubCategory::where('category_id', $request->input('category_select'))->get(),
            'suppliers' => Supplier::all()
        ]);
      }
      return view('product.category_list', [
             'categories' => Category::all()
      ]); 
    }




    public function single(Product $product, Request $request){
        return view('product.single', [
            'product' => $product
        ]);
    }
    public function edit(Product $product, Request $request){
        if($request->isMethod('POST')){  
           $validatedData = $request->validate([
           'title' => 'required|max:255',
           'SKU' => 'required',
           'brand_owner' => 'required',
           'image' => 'sometimes|file|image|max:5000',
           ]);
           
           $product->modified_by = Auth::user()->id;
           $product->title = $request->input('title');
           $product->SKU = $request->input('SKU');
           $product->brand_owner = $request->input('brand_owner');
           $product->description = $request->input('description');
           $product->purchase_UOM = $request->input('purchase_UOM');
           $product->save(); 

           if($request->has('image')){
              $product->image = 'images/products/'.$product->id."_".time().'.jpg';
              $product->save();
              $request->image->move('images/products/', $product->id."_".time().'.jpg');
              $img = Image::make(public_path('images/products/'.$product->id."_".time().'.jpg'))->crop(700, 700);
              $img->save();
           } 
 
      
            return redirect()->route('product.single', [
                'product' => $product->id
            ]);
        }
        return view('product.edit', [
            'product' => $product
        ]);
    }
    public function delete(Product $product, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            $product->delete();
            return redirect()->route('product.index');
        }
        return view('product.delete', [
            'product' => $product
        ]);
    }

    public function search(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'search_product' => 'required',
           ]);
         return view('product.index', [
            'products' => Product::where('title', 'LIKE', '%'.$request->search_product.'%')->paginate(15)
        ]);
       }//if post
      $search_result = array();
      $products = Product::where('title', 'LIKE', '%'.$request->term.'%')->get();
      if($products->count() == 0){
         $search_result[] = 'No match found';
      }else{
        $products->map(function($product)use(&$search_result){
           $product_obj = array();
           $product_obj['value'] = 'https://dev.smartpub.app-ic.co.uk/admin/product/'.$product->id;
           $product_obj['label'] = $product->title;
           $search_result[] = $product_obj;
        });         
        
      }

      return $search_result;
      
      
    }//end of search method   
}
