<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request; 
use App\Models\Category;
use App\Models\Pub;
use App\Models\Permission;
use App\Models\Stock;
use App\Models\Product;
use App\Models\Supplier;
use App\User;
use Auth;
use Hash;
use Redirect;

class PubController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
         return view('pub.index', [
            'pubs' => Pub::where('is_active', 1)
                          ->orderBy('name', 'asc')
                          ->paginate(15),
        ]);
    }
    
    public function new(Request $request){
        if($request->isMethod('POST')){
            
            $validatedData = $request->validate([
                'pub_name' => ['required', 'string', 'max:100'],
                'address_line1' => ['required', 'string', 'max:255'],
                'city' => ['required', 'string', 'max:100'],
                'postcode' => ['required', 'string', 'max:50'],
                'country' => ['required', 'string', 'max:100'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:pubs'],
            ]);

            $pub = new Pub;
            $pub->modified_by = Auth::user()->id;
            $pub->name = $request->input('pub_name');
            $pub->contact_person = $request->input('contact_person');
            $pub->contact_number = $request->input('contact_number');
            $pub->address_line1 = $request->input('address_line1');
            $pub->address_line2 = $request->input('address_line2');
            $pub->city = $request->input('city');
            $pub->postcode = $request->input('postcode');
            $pub->country = $request->input('country');
            $pub->email = $request->input('email');
            $pub->eircode = $request->input('eircode');
            $pub->account_no = $request->input('account_no');
            $pub->HIL_payer_no = $request->input('HIL_payer_no');
            $pub->CBL_acc_no = $request->input('CBL_acc_no');
            $pub->save();


            DB::table('product_pub')->insert([
                         'product_id' => 1,
                         'pub_id' => $pub->id, 
                      ]);

            $is_found = Stock::where('pub_id', $pub->id)->where('is_active', 1)->first();
            if($is_found){$is_found->update(['is_active' => 0]);}

            $stock = new Stock;
            $stock->pub_id = $pub->id;
            $stock->save();
              

            $products =  Pub::where('id', $pub->id)->first()->products;
               
            foreach($products as $product){
                    DB::table('product_stock')->insert([
                        'product_id' => $product->id,
                        'stock_id' => $stock->id,
                        'qty' => 0, 
                      ]);
                   }


            
            return redirect()->route('pub.single', [
                'pub' => $pub->id,
                'users' => $pub->users
            ]);
        }
        return view('pub.new', [
            'pub' => new Pub
        ]);
    }
    public function single(Pub $pub, Request $request){
        $user_permissions = array();
        $favourite_suppliers = array();
        $favourite_products = array();
        $suppliers = Supplier::where('is_active', 1)->get();
        $product_pub = DB::table('product_pub')->where('pub_id', $pub->id)->get();
        
        $product_pub->map(function($pro_pub)use(&$favourite_products, $suppliers, &$favourite_suppliers){
          $product_obj = array();
          $product = Product::where('id', $pro_pub->product_id)->first();
          if($product){
              $product_obj['id'] = $product->id;
              $product_obj['title'] = $product->title;
              $product_obj['SKU'] = $product->SKU;
              $product_obj['purchase_UOM'] = $product->purchase_UOM;
              $favourite_products[] = $product_obj;
            }

          $suppliers->map(function($supplier)use(&$favourite_suppliers, $pro_pub){
             $supplier_obj = array();
             if($pro_pub->supplier_id == $supplier->id){
                $supplier_obj['id'] = $supplier->id;
                $supplier_obj['name'] = $supplier->name;
                $supplier_obj['supplier_code'] = $supplier->supplier_code;
                $supplier_obj['REF'] = $supplier->REF;
                $supplier_obj['integrated'] = $supplier->integrated;
                $favourite_suppliers[] = $supplier_obj;
               }//if supplier
          });//map supplier
        });//map pro_pub
        $pub->users->map(function($user)use(&$user_permissions, $pub){
            $user_obj = array();
            $user_obj['pub_id'] = $pub->id;
            $user_obj['user_id'] = $user->id;
            $user_obj['name'] = $user->name;
            $user_obj['known_as'] = $user->known_as;
            $user_obj['role'] = $user->role->title;
            $user_obj['permissions'] = DB::table('permission_user')
                                         ->where('pub_id', $pub->id)
                                         ->where('user_id', $user->id)
                                         ->get();
            $user_obj['is_active'] = DB::table('pub_user')
                                        ->where('pub_id', $pub->id)
                                        ->where('user_id', $user->id)
                                        ->first()->is_active;                             
            $user_permissions[] = $user_obj;
        });

        return view('pub.single', [
            'pub' => $pub,
            'user_permissions' => $user_permissions,
            'products' => $favourite_products,
            'suppliers' => $favourite_suppliers
        ]);
    }
    public function edit(Pub $pub, Request $request){
        if($request->isMethod('POST')){
           $validatedData = $request->validate([
                'pub_name' => ['required', 'string', 'max:100'],
                'address_line1' => ['required', 'string', 'max:255'],
                'city' => ['required', 'string', 'max:100'],
                'postcode' => ['required', 'string', 'max:50'],
                'country' => ['required', 'string', 'max:100'],
                'email' => ['required', 'string', 'email', 'max:255'],
            ]);
             
            $pub->modified_by = Auth::user()->id;
            $pub->name = $request->input('pub_name');
            $pub->contact_person = $request->input('contact_person');
            $pub->contact_number = $request->input('contact_number');
            $pub->address_line1 = $request->input('address_line1');
            $pub->address_line2 = $request->input('address_line2');
            $pub->city = $request->input('city');
            $pub->postcode = $request->input('postcode');
            $pub->country = $request->input('country');
            $pub->eircode = $request->input('eircode');
            $pub->account_no = $request->input('account_no');
            $pub->HIL_payer_no = $request->input('HIL_payer_no');
            $pub->CBL_acc_no = $request->input('CBL_acc_no');
            $pub->save();

            return redirect()->route('pub.single', [
                'pub' => $pub->id
            ]);
        }
        return view('pub.edit', [
            'pub' => $pub
        ]);
    }
     public function add_new_pub_manager_submit(Request $request){
            $validatedData = $request->validate([
                'pub_select' => ['required'],
                'user_type' => ['required'],
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
                'password' => [
                            'required',
                            'confirmed',
                            'string',
                            'min:10',             // must be at least 10 characters in length
                            'regex:/[a-z]/',      // must contain at least one lowercase letter
                            'regex:/[A-Z]/',      // must contain at least one uppercase letter
                            'regex:/[0-9]/',      // must contain at least one digit
                            'regex:/[@$!%*#?&]/', // must contain a special character
                          ],
            ]);
            $role_id = 3;
            if($request->input('user_type') == 'pub_user'){$role_id = 4;}
            if($request->input('user_type') == 'pub_admin'){$role_id = 3;}
            $pub_manager = new User;
            $pub_manager->name = $request->input('name');
            $pub_manager->known_as = $request->input('known_as');
            $pub_manager->phone = $request->input('phone');
            $pub_manager->role_id = $role_id;
            $pub_manager->email = $request->input('email');
            $pub_manager->password = Hash::make($request->input('password'));
            $pub_manager->save();


            //first check if the pub user is in the db
            //if not, insert
            $check = DB::table('pub_user')
                        ->where('pub_id', $request->input('pub_select'))
                        ->where('user_id', $pub_manager->id)
                        ->first();
              if(!$check){
                 DB::table('pub_user')->insert([
                  'pub_id' => $request->input('pub_select'),
                  'user_id' => $pub_manager->id
                 ]);
                }            

             return redirect()->route('pub_manager.index');
        }
        
        
    public function add_new_pub_manager_form(Request $request){
         return view('pub.new_manager', [
                'selected_pub_id' => $request->input('pub_id')
            ]);
    }


    public function edit_permission_form(Request $request){
      $user_id = $request->input('user_id');
      $pub_id = $request->input('pub_id');
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $permissions = Permission::get();
      $processed_permissions = array();

      $permissions->map(function($permission)use(&$processed_permissions, $granted_permissions, $user_id, $pub_id){
        $is_granted = false;
        foreach($granted_permissions as $granted){
            if($granted->permission_id == $permission->id){
                $is_granted = true;
            }
        }
        $permission_obj = array();
        $permission_obj['id'] = $permission->id;
        $permission_obj['title'] = $permission->title;
        $permission_obj['is_granted'] = $is_granted;
        $processed_permissions[] = $permission_obj;
      });



      return view('pub.edit_permission', compact('user_id', 'pub_id', 'processed_permissions'));
    }


    public function edit_permission(Request $request){
      $user_id = $request->input('user_id');
      $pub_id = $request->input('pub_id');
      //is_granted
      //is_input
      //is_granted BUT not is_input ---> delete from permission_user table
      //is_input BUT not is_granted ---> insert into permission_user table
      //is_input but AND is_granted ---> do nothing
      $granted_permissions = DB::table('permission_user')
                                ->where('pub_id', $pub_id)
                                ->where('user_id', $user_id)
                                ->get();
      
      $permissions = Permission::get();
      
      $permissions->map(function($permission)use($granted_permissions, $user_id, $pub_id, $request){
        $is_granted = false;
        $is_input = false;
        foreach($granted_permissions as $granted){
            if($granted->permission_id == $permission->id){
                $is_granted = true;
            }
        }
        if($request->input($permission->title)){
            $is_input = true;
        }
        if($is_granted && !$is_input){
           DB::table('permission_user')
              ->where('pub_id', $pub_id)
              ->where('user_id', $user_id)
              ->where('permission_id', $permission->id)
              ->delete(); 
        }
        if(!$is_granted && $is_input){
           DB::table('permission_user')->insert(
            [
              'pub_id' => $pub_id,
              'user_id' => $user_id,
              'permission_id' => $permission->id,
              'permission' => $permission->title
            ]); 
        }
      });
      return Redirect::to('pub/'.$pub_id); 

    }//end of method
    
    public function user_status(Request $request){
      DB::table('pub_user')
         ->where('pub_id', $request->input('pub_id'))
         ->where('user_id', $request->input('user_id'))
         ->update([
            'is_active' => $request->input('is_active')
         ]);

      return Redirect::to('pub/'.$request->input('pub_id'));   
    }


    public function delete(Pub $pub, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            $pub->delete();
            return redirect()->route('pub.index');
        }
        return view('pub.delete', [
            'pub' => $pub
        ]);
    }

    public function search(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'search_pub' => 'required',
           ]);
         return view('pub.index', [
            'pubs' => Pub::where('is_active', 1)
                          ->where('name', 'LIKE', '%'.$request->search_pub.'%')
                          ->orderBy('name', 'asc')
                          ->paginate(15)
        ]);
       }//if post
      $search_result = array();
      $results = Pub::where('name', 'LIKE', '%'.$request->term.'%')->get();
      if($results->count() == 0){
         $search_result[] = 'No match found';
      }else{
        $results->map(function($result)use(&$search_result){
           $result_obj = array();
           $result_obj['value'] = 'https://dev.smartpub.app-ic.co.uk/admin/pub/'.$result->id;
           $result_obj['label'] = $result->name;
           $search_result[] = $result_obj;
        });         
        
      }

      return $search_result;
      
      
    }//end of search method   
}
