<?php

namespace App\Http\Controllers;

use  Intervention\Image\Facades\Image; 
use App\Models\SubCategory;
use App\Models\Category;
use Illuminate\Http\Request;
use Auth;

class SubCategoryController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
         return view('sub_category.index', [
            'sub_categories' => SubCategory::orderBy('title', 'asc')->paginate(15)
        ]);
    }
    
    public function new(Request $request){
        if($request->isMethod('POST')){      
           $validatedData = $request->validate([
           'title' => 'required|unique:sub_categories|max:255',
           'image' => 'sometimes|file|image|max:5000',
           ]);
           
           $sub_category = new SubCategory;
           $sub_category->modified_by = Auth::user()->id;
           $sub_category->category_id = $request->input('category_id');
           $sub_category->title = $request->input('title');
           $sub_category->save();  


          if($request->has('image')){
              $sub_category->image = 'images/sub_categories/'.$sub_category->id."_".time().'.jpg';
              $sub_category->save();
              $request->image->move('images/sub_categories/', $sub_category->id."_".time().'.jpg');
              $img = Image::make(public_path('images/sub_categories/'.$sub_category->id."_".time().'.jpg'))->crop(700, 700);
              $img->save();
           } 


            return redirect()->route('sub_category.single', [
                'sub_category' => $sub_category->id
            ]);
        }
        return view('sub_category.new', [
            'sub_category' => new SubCategory,
            'categories' => Category::all(),
        ]);
    }
    public function single(SubCategory $sub_category, Request $request){
        return view('sub_category.single', [
            'sub_category' => $sub_category
        ]);
    }
    public function edit(SubCategory $sub_category, Request $request){
        if($request->isMethod('POST')){  
           $validatedData = $request->validate([
           'title' => 'required|max:255',
           'image' => 'sometimes|file|image|max:5000',
           ]);
           
           $sub_category->modified_by = Auth::user()->id;
           $sub_category->category_id = $request->input('category_id');
           $sub_category->title = $request->input('title');
           $sub_category->save();  


           if($request->has('image')){
              $sub_category->image = 'images/sub_categories/'.$sub_category->id."_".time().'.jpg';
              $sub_category->save();
              $request->image->move('images/sub_categories/', $sub_category->id."_".time().'.jpg');
              $img = Image::make(public_path('images/sub_categories/'.$sub_category->id."_".time().'.jpg'))->crop(700, 700);
              $img->save();
           }

      
            return redirect()->route('sub_category.single', [
                'sub_category' => $sub_category->id
            ]);
        }
        return view('sub_category.edit', [
            'sub_category' => $sub_category,
            'categories' => Category::all(),
        ]);
    }
    public function delete(SubCategory $sub_category, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            $sub_category->remove();
            return redirect()->route('sub_category.index');
        }
        return view('sub_category.delete', [
            'sub_category' => $sub_category
        ]);
    }

    public function search(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'search_sub_category' => 'required',
           ]);
         return view('sub_category.index', [
            'sub_categories' => SubCategory::where('title', 'LIKE', '%'.$request->search_sub_category.'%')
                                            ->orderBy('title', 'desc')->paginate(15)              
        ]);
       }//if post
      $search_result = array();
      $results = SubCategory::where('title', 'LIKE', '%'.$request->term.'%')->get();
      if($results->count() == 0){
         $search_result[] = 'No match found';
      }else{
        $results->map(function($result)use(&$search_result){
           $result_obj = array();
           $result_obj['value'] = 'https://dev.smartpub.app-ic.co.uk/admin/sub_category/'.$result->id;
           $result_obj['label'] = $result->title;
           $search_result[] = $result_obj;
        });         
        
      }

      return $search_result;
      
      
    }//end of search method   
}
