<?php

namespace App\Http\Controllers;
 
use App\Models\Category;
use App\Models\Supplier;
use App\User;
use Illuminate\Http\Request;
use Auth;

class SupplierController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
         return view('supplier.index', [
            'suppliers' => Supplier::orderBy('name', 'asc')->paginate(15)
        ]);
    }

     public function desc(Request $request){
         return view('supplier.index_desc', [
            'suppliers' => Supplier::orderBy('name', 'desc')->paginate(15)
        ]);
    }
    
    public function new(Request $request){
        if($request->isMethod('POST')){
            
            $validatedData = $request->validate([
                'name' => ['required', 'string', 'max:100', 'unique:suppliers'],
                'supplier_code' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:suppliers'],
                'REF' => ['required', 'string', 'max:100'],
                
            ]);

            $supplier = new Supplier;
            $supplier->modified_by = Auth::user()->id;
            $supplier->name = $request->input('name');
            $supplier->supplier_code = $request->input('supplier_code');
            $supplier->email = $request->input('email');
            $supplier->REF = $request->input('REF');
            $supplier->save();


            return redirect()->route('supplier.single', [
                'supplier' => $supplier->id
            ]);
        }
        return view('supplier.new', [
            'supplier' => new Supplier
        ]);
    }
    public function single(Supplier $supplier, Request $request){
        return view('supplier.single', [
            'supplier' => $supplier
        ]);
    }
    public function edit(Supplier $supplier, Request $request){
        if($request->isMethod('POST')){
           $validatedData = $request->validate([
                'name' => ['required', 'string', 'max:100'],
                'supplier_code' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255'],
                'REF' => ['required', 'string', 'max:100'],
            ]);
             
            $data = array();
            $data['modified_by'] = Auth::user()->id;
            $data['name'] = $request->input('name');
            $data['supplier_code'] = $request->input('supplier_code');
            $data['email'] = $request->input('email');
            $data['REF'] = $request->input('REF');
            $supplier->update($data); 
            return redirect()->route('supplier.single', [
                'supplier' => $supplier->id
            ]);
        }
        return view('supplier.edit', [
            'supplier' => $supplier
        ]);
    }
    public function delete(Supplier $supplier, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            $supplier->delete();
            return redirect()->route('supplier.index');
        }
        return view('supplier.delete', [
            'supplier' => $supplier
        ]);
    }


    public function search(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'search_supplier' => 'required',
           ]);
         return view('supplier.index', [
            'suppliers' => Supplier::where('name', 'LIKE', '%'.$request->search_supplier.'%')
                                    ->orderBy('name', 'asc')->paginate(15)                                            
        ]);
       }//if post
      $search_result = array();
      $results = Supplier::where('name', 'LIKE', '%'.$request->term.'%')->get();
      if($results->count() == 0){
         $search_result[] = 'No match found';
      }else{
        $results->map(function($result)use(&$search_result){
           $result_obj = array();
           $result_obj['value'] = 'https://dev.smartpub.app-ic.co.uk/admin/supplier/'.$result->id;
           $result_obj['label'] = $result->name;
           $search_result[] = $result_obj;
        });         
        
      }

      return $search_result;
      
      
    }//end of search method   
}
