<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller; 
use App\Models\Category;
use App\User as Admin;
use Illuminate\Http\Request;
use Hash;
use Auth;

class AdminController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('SuperAdmin');
    }


     public function index(Request $request){
         return view('admin.index', [
            'admins' => Admin::where('role_id', 2)->orderBy('created_at', 'desc')->paginate(15)
        ]);
    }
    
    public function new(Request $request){
        if($request->isMethod('POST')){
            
            $validatedData = $request->validate([
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
                'password' => [
                            'required',
                            'confirmed',
                            'string',
                            'min:10',             // must be at least 10 characters in length
                            'regex:/[a-z]/',      // must contain at least one lowercase letter
                            'regex:/[A-Z]/',      // must contain at least one uppercase letter
                            'regex:/[0-9]/',      // must contain at least one digit
                            'regex:/[@$!%*#?&]/', // must contain a special character
                          ],
            ]);
             
            $admin = new Admin;
            $admin->name = $request->input('name');
            $admin->known_as = $request->input('known_as');
            $admin->phone = $request->input('phone');
            $admin->role_id = 2;
            $admin->email = $request->input('email');
            $admin->password = Hash::make($request->input('password'));
            $admin->save();


            return redirect()->route('admin.single', [
                'admin' => $admin->id
            ]);
        }
        return view('admin.new', [
            'admin' => new Admin,
        ]);
    }
    public function single(Admin $admin, Request $request){
        return view('admin.single', [
            'admin' => $admin
        ]);
    }
    public function edit(Admin $admin, Request $request){
        if($request->isMethod('POST')){
            $validatedData = $request->validate([
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255'],
            ]);
             
            

            $admin->name = $request->input('name');
            $admin->known_as = $request->input('known_as');
            $admin->phone = $request->input('phone');
            $admin->role_id = 2;
            $admin->email = $request->input('email');
            $admin->save();

            
            return redirect()->route('admin.single', [
                'admin' => $admin->id
            ]);
        }
        return view('admin.edit', [
            'admin' => $admin,
        ]);
    }
    public function delete(Admin $admin, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            $admin->delete();
            return redirect()->route('admin.index');
        }
        return view('admin.delete', [
            'admin' => $admin
        ]);
    }   
}
