<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller; 
use Illuminate\Support\Facades\DB;
use App\Models\Category;
use App\Models\Pub;
use App\Models\Permission;
use App\User as PubAdmin;
use Illuminate\Http\Request;
use Session;
use Hash;
use Auth;

class PubAdminController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
          $pub_admins = PubAdmin::where('role_id', 4)
                                 ->orderBy('name', 'asc')
                                 ->paginate(15);
               
         return view('pub_admin.index', ['pub_admins' => $pub_admins]);
    }
    
    public function new(Request $request){
        if($request->isMethod('POST')){
            $validatedData = $request->validate([
                'pub_select' => ['required'],
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
                'password' => [
                            'required',
                            'confirmed',
                            'string',
                            'min:10',             // must be at least 10 characters in length
                            'regex:/[a-z]/',      // must contain at least one lowercase letter
                            'regex:/[A-Z]/',      // must contain at least one uppercase letter
                            'regex:/[0-9]/',      // must contain at least one digit
                            'regex:/[@$!%*#?&]/', // must contain a special character
                          ],
            ]);
            //return $request;
            
            $pub_admin = new PubAdmin;
            $pub_admin->name = $request->input('name');
            $pub_admin->known_as = $request->input('known_as');
            $pub_admin->phone = $request->input('phone');
            $pub_admin->role_id = 4;
            $pub_admin->email = $request->input('email');
            $pub_admin->password = Hash::make($request->input('password'));
            $pub_admin->save();

            
            
            foreach($request->input('pub_select') as $pub_id){
              //first check if the pub user is in the db
              //if not, insert
              //deal with permissions
              $user_id = $pub_admin->id;
              $check = DB::table('pub_user')
                          ->where('pub_id', $pub_id)
                          ->where('user_id', $user_id)
                          ->first();
              if(!$check){
                 DB::table('pub_user')->insert([
                  'pub_id' => $pub_id,
                  'user_id' => $user_id
                 ]);
                }
              //is_granted
              //is_input
              //is_granted BUT not is_input ---> delete from permission_user table
              //is_input BUT not is_granted ---> insert into permission_user table
              //is_input but AND is_granted ---> do nothing
              $granted_permissions = DB::table('permission_user')
                                        ->where('pub_id', $pub_id)
                                        ->where('user_id', $user_id)
                                        ->get();
              
              $permissions = Permission::get();
              
              $permissions->map(function($permission)use($granted_permissions, $user_id, $pub_id, $request){
                $is_granted = false;
                $is_input = false;
                foreach($granted_permissions as $granted){
                    if($granted->permission_id == $permission->id){
                        $is_granted = true;
                    }
                }
                if($request->input($permission->title)){
                    $is_input = true;
                }
                if($is_granted && !$is_input){
                   DB::table('permission_user')
                      ->where('pub_id', $pub_id)
                      ->where('user_id', $user_id)
                      ->where('permission_id', $permission->id)
                      ->delete(); 
                }
                if(!$is_granted && $is_input){
                   DB::table('permission_user')->insert(
                    [
                      'pub_id' => $pub_id,
                      'user_id' => $user_id,
                      'permission_id' => $permission->id,
                      'permission' => $permission->title
                    ]); 
                }
              });              
            }//end of foreach pub


            return redirect()->route('pub_admin.single', [
                'pub_admin' => $pub_admin->id,
            ]);
        }//if post
        
        if(Auth::user()->role->title == 'PubManager'){
           $pubs = Pub::where('id', Auth::user()->pub_id)
                       ->where('is_active', 1)
                       ->paginate(15);
          }else{$pubs = Pub::where('is_active', 1)->get();}


        return view('pub_admin.new', [
                                      'pub_admin' => new PubAdmin,
                                      'pubs' => $pubs,
                                      'permissions' => Permission::get()
                                     ]);
    }//end of method
    public function single(PubAdmin $pub_admin, Request $request){
        $user_pubs = $pub_admin->pubs;
        $pubs = array();
        $user_pubs->map(function($pub)use(&$pubs, $pub_admin){
            $pub_obj = array();
            $pub_obj['id'] = $pub->id;
            $pub_obj['name'] = $pub->name;
            $pub_obj['is_active'] = DB::table('pub_user')
                                         ->where('pub_id', $pub->id)
                                         ->where('user_id', $pub_admin->id)
                                         ->first()->is_active;
                                         
            $pub_obj['permissions'] = DB::table('permission_user')
                                         ->where('pub_id', $pub->id)
                                         ->where('user_id', $pub_admin->id)
                                         ->get();
            $pubs[] = $pub_obj; 
        });
        //return $pubs;
        return view('pub_admin.single', [
            'pub_admin' => $pub_admin,
            'pubs' => $pubs
        ]);
    }
    public function edit(PubAdmin $pub_admin, Request $request){
        if($request->isMethod('POST')){
           $validatedData = $request->validate([
                'pub_select' => ['required'],
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
            ]);
             
            
            $pub_admin->name = $request->input('name');
            $pub_admin->known_as = $request->input('known_as');
            $pub_admin->phone = $request->input('phone');
            $pub_admin->save(); 
            
            
           //first delete all the pubs for that user then add them from input
            DB::table('pub_user')
                ->where('user_id', $pub_admin->id)
                ->delete();

           foreach($request->input('pub_select') as $pub_id){
              //first check if the pub user is in the db
              //if not, insert
              $user_id = $pub_admin->id;
              $check = DB::table('pub_user')
                          ->where('pub_id', $pub_id)
                          ->where('user_id', $user_id)
                          ->first();
              if(!$check){
                 DB::table('pub_user')->insert([
                  'pub_id' => $pub_id,
                  'user_id' => $user_id
                 ]);
                }            
              //is_granted
              //is_input
              //is_granted BUT not is_input ---> delete from permission_user table
              //is_input BUT not is_granted ---> insert into permission_user table
              //is_input but AND is_granted ---> do nothing
              $granted_permissions = DB::table('permission_user')
                                        ->where('pub_id', $pub_id)
                                        ->where('user_id', $user_id)
                                        ->get();
              
              $permissions = Permission::get();
              
              $permissions->map(function($permission)use($granted_permissions, $user_id, $pub_id, $request){
                $is_granted = false;
                $is_input = false;
                foreach($granted_permissions as $granted){
                    if($granted->permission_id == $permission->id){
                        $is_granted = true;
                    }
                }
                if($request->input($permission->title)){
                    $is_input = true;
                }
                if($is_granted && !$is_input){
                   DB::table('permission_user')
                      ->where('pub_id', $pub_id)
                      ->where('user_id', $user_id)
                      ->where('permission_id', $permission->id)
                      ->delete(); 
                }
                if(!$is_granted && $is_input){
                   DB::table('permission_user')->insert(
                    [
                      'pub_id' => $pub_id,
                      'user_id' => $user_id,
                      'permission_id' => $permission->id,
                      'permission' => $permission->title
                    ]); 
                }
              });


            }//end of foreach
            

            return redirect()->route('pub_admin.single', [
                'pub_admin' => $pub_admin->id
            ]);
        }//end of if post
        
          
        
        return view('pub_admin.edit', [
            'pub_admin' => $pub_admin,
            'pubs' => Pub::where('is_active', 1)->get(),
            'permissions' => Permission::get()
        ]);
    }
    public function delete(PubAdmin $pub_admin, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            DB::table('pub_user')
                ->where('user_id', $pub_admin->id)
                ->delete();
            DB::table('permission_user')
                ->where('user_id', $pub_admin->id)
                ->delete();
            DB::table('setting_user')
                ->where('user_id', $pub_admin->id)
                ->delete();
            DB::table('terms_and_condition_user')
                ->where('user_id', $pub_admin->id)
                ->delete();    
            $pub_admin->delete();
            return redirect()->route('pub_admin.index');
        }
        return view('pub_admin.delete', [
            'pub_admin' => $pub_admin
        ]);
    }


    public function search(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'search_pub_user' => 'required',
           ]);
         
         $pub_admins = PubAdmin::where('role_id', 4)
                                 ->where('name', 'LIKE', '%'.$request->search_pub_user.'%')
                                 ->orderBy('name', 'asc')
                                 ->paginate(15);
               
         return view('pub_admin.index', ['pub_admins' => $pub_admins]); 
       }//if post
      $search_result = array();
      $results = PubAdmin::where('name', 'LIKE', '%'.$request->term.'%')->where('role_id', 4)->get();
      if($results->count() == 0){
         $search_result[] = 'No match found';
      }else{
        $results->map(function($result)use(&$search_result){
           $result_obj = array();
           $result_obj['value'] = 'https://dev.smartpub.app-ic.co.uk/admin/pub_user/'.$result->id;
           $result_obj['label'] = $result->name;
           $search_result[] = $result_obj;
        });         
        
      }

      return $search_result;
      
      
    }//end of search method   
}
