<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\DB; 
use App\Models\Category;
use App\Models\Pub;
use App\User as PubManager;
use Illuminate\Http\Request;
use Hash;
use Auth;

class PubManagerController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware('Admin');
    }


     public function index(Request $request){
         return view('pub_manager.index', [
            'pub_managers' => PubManager::where('role_id', 3)->orderBy('name', 'asc')->paginate(15)
        ]);
    }
    
    public function new(Request $request, Pub $pub){
        if($request->isMethod('POST')){
            
            $validatedData = $request->validate([
                'pub_select' => ['required'],
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
                'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
                'password' => [
                            'required',
                            'confirmed',
                            'string',
                            'min:10',             // must be at least 10 characters in length
                            'regex:/[a-z]/',      // must contain at least one lowercase letter
                            'regex:/[A-Z]/',      // must contain at least one uppercase letter
                            'regex:/[0-9]/',      // must contain at least one digit
                            'regex:/[@$!%*#?&]/', // must contain a special character
                          ],
            ]);

            $pub_manager = new PubManager;
            $pub_manager->name = $request->input('name');
            $pub_manager->known_as = $request->input('known_as');
            $pub_manager->phone = $request->input('phone');
            $pub_manager->role_id = 3;
            $pub_manager->email = $request->input('email');
            $pub_manager->password = Hash::make($request->input('password'));
            $pub_manager->save();


            foreach($request->input('pub_select') as $pub_id){
              //first check if the pub user is in the db
              //if not, insert
              $check = DB::table('pub_user')
                          ->where('pub_id', $pub_id)
                          ->where('user_id', $pub_manager->id)
                          ->first();
              if(!$check){
                 DB::table('pub_user')->insert([
                  'pub_id' => $pub_id,
                  'user_id' => $pub_manager->id
                 ]);
                }            
            }//end of foreach


            return redirect()->route('pub_manager.single', [
                'pub_manager' => $pub_manager->id
            ]);
        }
        
       
        return view('pub_manager.new', [
            'pub_manager' => new PubManager,
            'pubs' => Pub::where('is_active', 1)->orderBy('name', 'asc')->get()
        ]);
    }
    public function single(PubManager $pub_manager, Request $request){
        $user_pubs = $pub_manager->pubs;
        $pubs = array();
        $user_pubs->map(function($pub)use(&$pubs, $pub_manager){
            $pub_obj = array();
            $pub_obj['id'] = $pub->id;
            $pub_obj['name'] = $pub->name;
            $pub_obj['is_active'] = DB::table('pub_user')
                                         ->where('pub_id', $pub->id)
                                         ->where('user_id', $pub_manager->id)
                                         ->first()->is_active;
                                         
            $pubs[] = $pub_obj; 
        });
        return view('pub_manager.single', [
            'pub_manager' => $pub_manager,
            'pubs' => $pubs
        ]);
    }
    public function edit(PubManager $pub_manager, Request $request){
        if($request->isMethod('POST')){

            $validatedData = $request->validate([
                'pub_select' => ['required'],
                'name' => ['required', 'string', 'max:255'],
                'known_as' => ['required', 'string', 'max:255'],
                'phone' => ['required', 'string', 'max:255'],
            ]);
             
            $pub_manager->name = $request->input('name');
            $pub_manager->known_as = $request->input('known_as');
            $pub_manager->phone = $request->input('phone');
            $pub_manager->save(); 
            
            //first delete all the pubs for that user then add them from input
            DB::table('pub_user')
                ->where('user_id', $pub_manager->id)
                ->delete();

            foreach($request->input('pub_select') as $pub_id){
              //first check if the pub user is in the db
              //if not, insert
              $check = DB::table('pub_user')
                          ->where('pub_id', $pub_id)
                          ->where('user_id', $pub_manager->id)
                          ->first();
              if(!$check){
                 DB::table('pub_user')->insert([
                  'pub_id' => $pub_id,
                  'user_id' => $pub_manager->id
                 ]);
                }            
            }//end of foreach    


            return redirect()->route('pub_manager.single', [
                'pub_manager' => $pub_manager->id
            ]);
        }
        return view('pub_manager.edit', [
            'pub_manager' => $pub_manager,
            'pubs' => Pub::where('is_active', 1)->orderBy('name', 'asc')->get()
        ]);
    }
    public function delete(PubManager $pub_manager, Request $request){
        if($request->isMethod('POST') || $request->isMethod('DELETE')){
            DB::table('pub_user')
                ->where('user_id', $pub_manager->id)
                ->delete();
            DB::table('permission_user')
                ->where('user_id', $pub_manager->id)
                ->delete();
            DB::table('setting_user')
                ->where('user_id', $pub_manager->id)
                ->delete();
            DB::table('terms_and_condition_user')
                ->where('user_id', $pub_manager->id)
                ->delete();            
            $pub_manager->delete();
            return redirect()->route('pub_manager.index');
        }
        return view('pub_manager.delete', [
            'pub_manager' => $pub_manager
        ]);
    }


    public function search(Request $request){
      if($request->isMethod('POST')){
         $validatedData = $request->validate([
           'search_pub_admin' => 'required',
           ]);
         
               
         return view('pub_manager.index', [
            'pub_managers' => PubManager::where('role_id', 3)
                                         ->where('name', 'LIKE', '%'.$request->search_pub_admin.'%')
                                         ->orderBy('name', 'asc')->paginate(15)
        ]); 
       }//if post
      $search_result = array();
      $results = PubManager::where('name', 'LIKE', '%'.$request->term.'%')->where('role_id', 3)->get();
      if($results->count() == 0){
         $search_result[] = 'No match found';
      }else{
        $results->map(function($result)use(&$search_result){
           $result_obj = array();
           $result_obj['value'] = 'https://dev.smartpub.app-ic.co.uk/admin/pub_admin/'.$result->id;
           $result_obj['label'] = $result->name;
           $search_result[] = $result_obj;
        });         
        
      }

      return $search_result;
      
      
    }//end of search method   
}
