<?php

namespace App;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
                             'role_id',
                             'supplier_id',
                             'pub_id',
                             'name',
                             'known_as',
                             'phone', 
                             'email', 
                             'password',
                          ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];


    public function metadata(){
        return $this->hasMany('App\Models\UserMetadata');
    }


    public function feedback(){
        return $this->hasMany('App\Models\Feedback');
    }

    public function order(){
        return $this->hasMany('App\Models\Order');
    }


    public function role(){
        return $this->belongsTo('App\Models\Role', 'role_id', 'id');
    }

    public function supplier(){
        return $this->belongsTo('App\Models\Supplier', 'supplier_id', 'id');
    }

    public function termsAndCondision(){
        return $this->belongsToMany('App\Models\TermsAndCondision');
    }
    
    public function setting(){
        return $this->belongsToMany('App\Models\Setting');
    }
    
    public function pubs(){
        return $this->belongsToMany('App\Models\Pub');
    }

    public function permissions(){
        return $this->belongsToMany('App\Models\Permission');
    }


}
