<?php

/** @var \Illuminate\Database\Eloquent\Factory $factory */

use App\User;
use App\Models\Pub;
use App\Models\Supplier;
use App\Models\Product;
use App\Models\Stock;
use App\Models\Feedback;
use Faker\Generator as Faker;
use Illuminate\Support\Str;

/*
|--------------------------------------------------------------------------
| Model Factories
|--------------------------------------------------------------------------
|
| This directory should contain each of the model factory definitions for
| your application. Factories provide a convenient way to generate new
| model instances for testing / seeding your application's database.
|
*/

$factory->define(User::class, function (Faker $faker) {
    return [
    	'name' => $faker->name,
        'known_as' => $faker->firstName,
        'phone' => $faker->phoneNumber,
        'role_id' => $faker->numberBetween($min = 3, $max = 4),
        'email' => $faker->unique()->safeEmail,
        'email_verified_at' => now(),
        'password' => '$2y$10$TKh8H1.PfQx37YgCzwiKb.KjNyWgaHb9cbcoQgdIVFlYg7B77UdFm', // secret
        'remember_token' => Str::random(10),
    ];
});


$factory->define(Pub::class, function (Faker $faker) {
    return [
    	'modified_by' => 1,
        'name' => $faker->company,
        'contact_person' => $faker->name,
        'contact_number' => $faker->phoneNumber,
        'address_line1' => $faker->address,
        'city' => $faker->city,
        'country' => $faker->country,
        'postcode' => $faker->postcode,
        'email' => $faker->unique()->safeEmail,
        'eircode' => $faker->numberBetween($min = 1000, $max = 9000),
        'account_no' => $faker->numberBetween($min = 1000, $max = 9000),
        'HIL_payer_no' => $faker->numberBetween($min = 1000, $max = 9000),
        'CBL_acc_no' => $faker->numberBetween($min = 1000, $max = 9000),
    ];
});



$factory->define(Supplier::class, function (Faker $faker) {
    return [
    	'modified_by' => 1,
        'name' => $faker->company,
        'supplier_code' => $faker->numberBetween($min = 1000, $max = 9000),
        'email' => $faker->unique()->safeEmail,
        'REF' => $faker->numberBetween($min = 1000, $max = 9000),
        'integrated' => 1,
    ];
});


$factory->define(Feedback::class, function (Faker $faker) {
    return [
        'user_id' => $faker->numberBetween($min = 2, $max = 1000),
        'rating' => $faker->numberBetween($min = 1, $max = 5),
        'comment' => $faker->text,
    ];
});


   //'image' => $faker->imageUrl($width = 640, $height = 480),

$factory->define(Product::class, function (Faker $faker) {
    return [
    	'sub_category_id' => $faker->numberBetween($min = 1, $max = 8),
    	'supplier_id' => $faker->numberBetween($min = 1, $max = 3),
    	'modified_by' => 1,
        'title' => $faker->unique()->word,
        'SKU' => $faker->numberBetween($min = 1000, $max = 9000),
        'brand_owner' => $faker->word,
        'purchase_UOM' => $faker->word,
        'description' => $faker->paragraph,
        'is_active' => 1, 
    ];
});








