<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::get('/', 'Auth\LoginController@showLoginForm');
Route::get('permission_denied', function(){
      return view('permission_denied');
     });





//_______________________________________________________________________________
//
//                             USERS
//
//_______________________________________________________________________________



Auth::routes();
Route::any('register', 'Auth\LoginController@showLoginForm');
Route::get('/home', 'HomeController@index')->name('home');

Route::prefix('admin')->group(function(){
  Route::any('/',              'User\AdminController@index')->name('admin.index');
  Route::any('new',            'User\AdminController@new')->name('admin.new');
  Route::any('{admin}',        'User\AdminController@single')->name('admin.single');
  Route::any('{admin}/edit',   'User\AdminController@edit')->name('admin.edit');
  Route::any('{admin}/delete', 'User\AdminController@delete')->name('admin.delete');
});

Route::prefix('pub_admin')->group(function(){
  Route::any('/',                    'User\PubManagerController@index')->name('pub_manager.index');
  Route::any('new',                  'User\PubManagerController@new')->name('pub_manager.new');
  Route::any('{pub_manager}',        'User\PubManagerController@single')->name('pub_manager.single');
  Route::any('{pub_manager}/edit',   'User\PubManagerController@edit')->name('pub_manager.edit');
  Route::any('{pub_manager}/delete', 'User\PubManagerController@delete')->name('pub_manager.delete');
});

Route::prefix('pub_user')->group(function(){
  Route::any('/',                  'User\PubAdminController@index')->name('pub_admin.index');
  Route::any('new',                'User\PubAdminController@new')->name('pub_admin.new');
  Route::any('{pub_admin}',        'User\PubAdminController@single')->name('pub_admin.single');
  Route::any('{pub_admin}/edit',   'User\PubAdminController@edit')->name('pub_admin.edit');
  Route::any('{pub_admin}/delete', 'User\PubAdminController@delete')->name('pub_admin.delete');
});



//_______________________________________________________________________________
//
//                  ACCOUNTS _____ PUBS _____ SUPPLIERS
//
//_______________________________________________________________________________



Route::prefix('pub')->group(function(){
  Route::any('/',                  'PubController@index')->name('pub.index');
  Route::any('new',                'PubController@new')->name('pub.new');
  Route::any('{pub}',              'PubController@single')->name('pub.single');
  Route::any('{pub}/edit',         'PubController@edit')->name('pub.edit');
  Route::any('{pub}/delete',       'PubController@delete')->name('pub.delete');
});

Route::prefix('supplier')->group(function(){
  Route::any('/',                 'SupplierController@index')->name('supplier.index');
  Route::any('desc',              'SupplierController@desc')->name('supplier.desc');
  Route::any('new',               'SupplierController@new')->name('supplier.new');
  Route::any('{supplier}',        'SupplierController@single')->name('supplier.single');
  Route::any('{supplier}/edit',   'SupplierController@edit')->name('supplier.edit');
  Route::any('{supplier}/delete', 'SupplierController@delete')->name('supplier.delete');
});

Route::post('add_new_pub_manager_submit', 'PubController@add_new_pub_manager_submit');
Route::get('add_new_pub_manager_form', 'PubController@add_new_pub_manager_form');
Route::post('edit_permission_form', 'PubController@edit_permission_form');
Route::post('edit_permission', 'PubController@edit_permission');
Route::post('pub_user_status', 'PubController@user_status');



//_______________________________________________________________________________
//
//                             CATEGORIES
//
//_______________________________________________________________________________



Route::prefix('category')->group(function(){
  Route::any('/',                 'CategoryController@index')->name('category.index');
  Route::any('new',               'CategoryController@new')->name('category.new');
  Route::any('{category}',        'CategoryController@single')->name('category.single');
  Route::any('{category}/edit',   'CategoryController@edit')->name('category.edit');
  Route::any('{category}/delete', 'CategoryController@delete')->name('category.delete');
});

Route::prefix('sub_category')->group(function(){
  Route::any('/',                     'SubCategoryController@index')->name('sub_category.index');
  Route::any('new',                   'SubCategoryController@new')->name('sub_category.new');
  Route::any('{sub_category}',        'SubCategoryController@single')->name('sub_category.single');
  Route::any('{sub_category}/edit',   'SubCategoryController@edit')->name('sub_category.edit');
  Route::any('{sub_category}/delete', 'SubCategoryController@delete')->name('sub_category.delete');
});



//_______________________________________________________________________________
//
//                             PRODUCTS
//
//_______________________________________________________________________________



Route::prefix('product')->group(function(){
  Route::any('/',                'Product\ProductController@index')->name('product.index');
  Route::any('desc',             'Product\ProductController@index_desc')->name('product.index_desc');
  Route::any('sku',              'Product\ProductController@sku')->name('product.sku');
  Route::any('brand_owner',      'Product\ProductController@brand_owner')->name('product.brand_owner');
  Route::post('new',             'Product\ProductController@new')->name('product.new');
  Route::any('category_list',    'Product\ProductController@category_list')->name('product.category_list');
  Route::any('{product}',        'Product\ProductController@single')->name('product.single');
  Route::any('{product}/edit',   'Product\ProductController@edit')->name('product.edit');
  Route::any('{product}/delete', 'Product\ProductController@delete')->name('product.delete');
});

Route::get('multiple_product_input', 'Product\MultipleInputProductController@multiple_input');
Route::any('bulk_upload', 'Product\MultipleInputProductController@bulk_upload');




//______________________________________________________________________________
//
//                                SEARCH
//
//______________________________________________________________________________

Route::any('search_product',      'Product\ProductController@search');
Route::any('search_sub_category', 'SubCategoryController@search');
Route::any('search_supplier',     'SupplierController@search');
Route::any('search_pub',          'PubController@search');
Route::any('search_pub_admin',    'User\PubManagerController@search');
Route::any('search_pub_user',     'User\PubAdminController@search');




//_______________________________________________________________________________
//
//                             STOCK
//
//_______________________________________________________________________________


Route::prefix('admin_stock')->group(function(){
  Route::any('/',              'Stock\TargetStockController@index')->name('admin_stock.index');
  Route::post('edit',          'Stock\TargetStockController@edit')->name('target_stock.edit');
  Route::any('{stock}/delete', 'Stock\TargetStockController@delete')->name('target_stock.delete');
});





//_______________________________________________________________________________
//
//                             ORDER
//
//_______________________________________________________________________________



Route::prefix('order')->group(function(){
  Route::any('/',              'Order\OrderController@current')->name('order.current');
  Route::any('new_product',    'Order\OrderController@new_product')->name('order.new_product');
  Route::any('confirm',        'Order\OrderController@confirm')->name('order.confirm');
  Route::any('send',           'Order\OrderController@confirm')->name('order.send');
  Route::post('update_qty',    'Order\OrderController@update_qty')->name('order.update_qty');
  Route::post('remove_product','Order\OrderController@remove_product')->name('order.remove_product');
  Route::get('history',        'Order\OrderController@history')->name('order.history');
  Route::any('{order}',        'Order\OrderController@single')->name('order.single');
});

Route::get('cancel_all_suborder/{product_id}','Order\OrderController@cancel_all_suborder');
Route::get('cancel_suborder/{suborder_id}','Order\OrderController@cancel_suborder');
Route::get('send_suborder/{suborder_id}','Order\OrderController@send_suborder');


Route::get('all_order',           'Order\AdminOrderController@order_history');
Route::get('current_order',       'Order\AdminOrderController@current_order');
Route::get('single_order/{id}',   'Order\AdminOrderController@single_order');



//_______________________________________________________________________________
//
//                             DELIVERIES
//
//_______________________________________________________________________________




Route::prefix('delivery')->group(function(){
  Route::any('/',              'Delivery\DeliveryController@index')->name('delivery.index');
  Route::post('update',        'Delivery\DeliveryController@update')->name('delivery.update');
});


//_______________________________________________________________________________
//
//                             SDOCS
//
//_______________________________________________________________________________


Route::prefix('sdoc')->group(function(){
  Route::any('/',              'Sdoc\SdocController@index')->name('admin_stock.index');
});



//_______________________________________________________________________________
//
//                             FEEDBACK
//
//_______________________________________________________________________________



Route::prefix('feedback')->group(function(){
  Route::any('/',                 'Feedback\FeedbackController@index')->name('feedback.index');
  Route::any('{feedback}',        'Feedback\FeedbackController@single')->name('feedback.single');
  Route::any('{feedback}/delete', 'Feedback\FeedbackController@delete')->name('feedback.delete');
});



//_______________________________________________________________________________
//
//                             T & C
//
//_______________________________________________________________________________



Route::prefix('t_and_c')->group(function(){
  Route::any('/',                'TermsAndConditionController@index')->name('t_and_c.index');
  Route::any('new',              'TermsAndConditionController@new')->name('t_and_c.new');
  Route::any('{t_and_c}',        'TermsAndConditionController@single')->name('t_and_c.single');
  Route::any('{t_and_c}/delete', 'TermsAndConditionController@delete')->name('t_and_c.delete');
});




//_______________________________________________________________________________
//
//                             Setting
//
//_______________________________________________________________________________


Route::prefix('setting')->group(function(){
  Route::any('/',                'SettingController@index')->name('setting.index');
  Route::any('new',              'SettingController@new')->name('setting.new');
  Route::any('{setting}',        'SettingController@single')->name('setting.single');
  Route::any('{setting}/delete', 'SettingController@delete')->name('setting.delete');
});









